@@ -259,48 +259,46 @@ parser.add_argument(
 parser.add_argument(
     "-o", "--output",
     dest="output_filename",
-    help="write configuration to named file instead of stdout; '%%' "
-         "can be used to substitute in the name of the device into "
-         "the filename")
+    help="write configuration to named file instead of stdout; '%%' can be "
+             "used to substitute in the name of the device into the filename")
 
 parser.add_argument(
     "-O", "--no-output",
     action="store_true",
-    help="generate the configuration but do not output it - useful to "
-         "test generation succeeds")
+    help="generate the configuration but do not output it - useful to test "
+             "generation succeeds")
 
 parser.add_argument(
     "-S", "--no-squash-blanklines",
     action="store_true",
     help="disable the squashing of leading and trailing, as well as "
-         "multiple, consecutive blank lines (prevent equivalent of "
-         "'| cat -s') in output")
+             "multiple, consecutive blank lines (prevent equivalent of '| "
+             "cat -s') in output")
 
 parser.add_argument(
     "-E", "--list-env",
     action="store_true",
     help="list filter and global helper functions and stop (this will "
-         "include the standard Jinja2, as well as the netaddr module "
-         "and custom ones)")
+             "include the standard Jinja2, as well as the netaddr module and "
+             "custom ones)")
 
 parser.add_argument(
     "-I", "--dump-inventory",
     action="store_true",
-    help="dump complete read inventory in YAML to stdout and stop, "
-         "without generating any configurations")
+    help="dump complete read inventory in YAML to stdout and stop, without "
+             "generating any configurations")
 
 parser.add_argument(
     "-F", "--dump-filepaths",
     action="store_true",
-    help="dump paths of files which are the source of each entry in "
-         "the inventory")
+    help="dump paths of files which are the source of each entry in the "
+             "inventory")
 
 parser.add_argument(
     "-U", "--dump-device",
     action="store_true",
-    help="dump resulting device definition in YAML to stdout, after "
-         "merging profiles and stop, without generating any "
-         "configurations")
+    help="dump resulting device definition in YAML to stdout, after merging "
+             "profiles and stop, without generating any configurations")
 
 parser.add_argument(
     "-d", "--define",
@@ -314,13 +312,21 @@ parser.add_argument(
     "-q", "--quiet",
     action="store_true",
     help="when generating configuration for multiple devices, don't "
-         "print the name of each device, as it's generated")
+             "print the name of each device, as it's generated")
 
 parser.add_argument(
     "-D", "--debug",
     action="store_true",
     help="enable debug mode")
 
+parser.add_argument(
+    "-R", "--raise-exception",
+    action="store_true",
+    help="when an exception is explicitly raised through a helper function, "
+            "raise the complete exception rather than catch it and just "
+            "print the message specific error message - this is useful if "
+            "the exception does not make it clear where it occurred")
+
 parser.add_argument(
     "devicename",
     nargs="*",
@@ -350,6 +356,7 @@ dump_device = args.dump_device
 quiet = args.quiet
 devicenames = args.devicename
 debug = args.debug
+raise_exception = args.raise_exception
 
 vars = {}
 for var, value in args.define:
@@ -441,9 +448,9 @@ env = jinja2.Environment(
 # add in the special warn(), raise() and assert() functions, as well as
 # some other functions we need, into the Jinja2 environment
 
-for helpers in [deep_helpers, helpers.helpers]:
-    for helper in helpers:
-        env.globals[helper] = helpers[helper]
+for helpers_dict in [deep_helpers, helpers.helpers]:
+    for helper in helpers_dict:
+        env.globals[helper] = helpers_dict[helper]
 
 
 # add in the netaddr library functions as additional filters
@@ -493,7 +500,25 @@ for devicename in devicenames:
     if (not quiet) and (len(devicenames) > 1):
         print(devicename)
 
-    complete_success &= generate_config(devicename)
+    try:
+        complete_success &= generate_config(devicename)
+
+    except helpers.HelperBaseException as e:
+        # one of the explicit helper exceptions has occurred
+
+
+        # if we're raising an exception anyway, do that; if not, just print
+        # the exception text in the form of a user-friendly error message
+
+        if raise_exception:
+            raise
+
+        print("error:", e)
+
+
+        # since something went wrong, we don't have 'complete success'
+
+        complete_success = False
 
 
 exit(0 if complete_success else 1)
